/* ---------------------------------------------
 * Analog Servo Control with Arduino
 * Author: Nico Teering
 * Version 1.1, June 2016
 * 
 * This source code or any part of it may NOT be used in any commercial product.
 * 
 * Please mail nico.teering@ziggo.nl if you have any comment or  
 * suggestions for improvement on this program.
 * ---------------------------------------------------
 * Version 1.1
 * 
 * New features:
 * Added option to save and restore servo settings
 * New method for changing to config mode
 * 
 * Changes:
 * For changing to config mode just connect USB-cable and press 'C'.
 * The 'config button' now functions as a 'save' button.
 * In config mode indicator led continuous on
 * Document command always executed when starting config mode
 * 
 * Bugs fixed:
 * The 'R'-Command was actually the 'C'-Command
 * Unable to select pin 12 as pin number. Now fixed
 * Servo kept rotating  when switch pressed continuously
 * Buddy pin changed from 2-13 to 2-12,14
*/

#include <EEPROM.h>
#include <Servo.h> 
//------------------------

const byte maxpins=19;
const String versie = "USB version 1.1";

struct PinItem 
{
  byte Langle;              //LOW angle             Yes
  byte Hangle;              //HIGH angle            Yes
  byte buddy;               //Buddy pin             Yes
  byte zwitch;              //switch nummer         yes
  byte mspeed;              //Inverted              Yes
  unsigned int nummer;      //Admin Number          Yes
  byte frogpin;             //Frog pin              Yes
  char RG;                  //r=rond, g=gerade      Yes
  byte type;                //=2, 4=frog, 3=led     Yes
  byte ledpin;              //ledpin                Yes 
  byte output;              //output         
  byte swrelay;             //switchangle 
  byte angle;               //Current angle  
  byte setpoint;            //Destination angle    
  boolean jip;              //Junction In Position   
  unsigned long lastmillis; //Time at last change
};
byte servpin;
byte numservos=0;
// Servo's only on: 2-12,14 
// Relay and leds only on 2-12,14-18
// A5 is analog input for switches
//unsigned int limits[12] = {66,168,262,350,434,515,594,670,746,824,903,980};
unsigned int limits[12] = {43,128,213,298,384,469,554,639,725,810,895,979};
PinItem allpins[maxpins];
Servo Servos[12];
byte switches[] = {0,0,0,0,0,0,0,0,0,0,0,0};
char cmdLH[maxpins];    //setting Low or H angle
int mode;               //config=255/normal=0
byte savout[maxpins];   //save last output
boolean changes=false;  //Wijzigingen?
byte curpin;            //Current pin being set
boolean valid;          //Entry valid?
int defspeed;           //default servo speed
char gencmd;            // command entered
unsigned long previousMillis;
unsigned long currentMillis;
boolean LED13state;
byte decnum;
boolean quick=false;
byte route;
int spin;
int zwitch;
unsigned long lastswmillis;
boolean zero;
boolean prod;
boolean pinsel = false;  //true if pin selected, false for generic mode 
byte ip[] = {0,0,0,1,2,3,4,5,6,7, 8, 9,10, 0,11,0,0,0,0,}; //convert pin number to Servo array index
//       pin     2 3 4 5 6 7 8 9 10 11 12 13 14
byte routes [12][12];
//           R   P
byte bounces [12][4];
//             P  A
unsigned int bounceangles[12];
int ten[]={1000,100,10,1};
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void setup()
{
  mode=EEPROM.read(500);
  if (mode!=0 && mode!=255)
  {
    mode=255;
    EEPROM.write(500,255);  // to fix uncontrolled changes of address 500
  }
  Serial.begin(9600);

  pinMode(LED_BUILTIN,OUTPUT);
  digitalWrite (LED_BUILTIN,LOW);
//  pinMode(A5,INPUT_PULLUP);
  pinMode(0,INPUT_PULLUP);

  start();
  
  defspeed=EEPROM.read(230);
  if (defspeed==255) defspeed=25;
  for (byte n=2; n<maxpins; n++)    //Read all settings from memory
  {
    allpins[n].Langle=EEPROM.read(1+11*n);     
    allpins[n].Hangle=EEPROM.read(2+11*n);
    allpins[n].mspeed=EEPROM.read(3+11*n);
    allpins[n].frogpin=EEPROM.read(4+11*n);
    allpins[n].zwitch=EEPROM.read(5+11*n);  
    allpins[n].buddy=EEPROM.read(6+11*n);  
    allpins[n].nummer=word(EEPROM.read(7+11*n),EEPROM.read(8+11*n));     
    allpins[n].RG=EEPROM.read(9+11*n);
    allpins[n].type=EEPROM.read(10+11*n);
    if (allpins[n].type==2 && allpins[n].RG==255) allpins[n].RG='X';
    allpins[n].ledpin=EEPROM.read(11+11*n);
    allpins[n].jip=true;
    if (allpins[n].nummer==65535) allpins[n].nummer=0;

    pinMode(n,OUTPUT);

    if (allpins[n].type==2 && !quick) 
    {
      servpin=ip[n];
//      bounceangles[servpin]=word(EEPROM.read(300+2*servpin),EEPROM.read(301+2*servpin)); 
//      if (bounceangles[servpin]==65535) bounceangles[servpin]=0;
//      cvtbounce();  
      allpins[n].swrelay=(allpins[n].Hangle+allpins[n].Langle)/2;
      switches[allpins[n].zwitch-1]=n;
      allpins[n].angle=allpins[n].swrelay;
      allpins[n].output=EEPROM.read(270+n); 
      if (allpins[n].output==255 || mode==255) allpins[n].output=1;
      Servos[servpin].attach(n); 
      setanglen(n);
      Servos[servpin].detach();
    }
  }

  if (!quick)
  {
    for (byte r=0; r<12; r++)
    {
      for (byte w=0; w<12; w++) 
      {
       spin=EEPROM.read(600+w+(12*r));
       if (spin>140) routes[r][w]=0; else  routes[r][w]=spin;
      }
    }
  }

//----------------------Configuration setup
  if (mode==255)
  {
    if (quick)
    {
      Serial.print(F("\nDo you want to do a quick configuration ? Y/N: "));
      if (askyn()=='Y') quickconfig();
    } 
    else document();

    digitalWrite (LED_BUILTIN,HIGH);
/*
    for (int n=2;n<maxpins-3;n++)  //force servo's Low
    {
      if (allpins[n].type==2)
      {
        allpins[n].angle=allpins[n].swrelay;
        allpins[n].output=1;
        Servos[servpin].attach(n); 
        setanglen(n);
        Servos[servpin].detach();
      }
    }
*/
  }
  if (mode==0) while (Serial.available()) Serial.read(); //empty buffer
  
} // end setup
//==================================================================

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void loop()
{
  if (mode==0) 
  {
    do
    {
//      processroute();
      processwitch();
    } while (mode==0);
    start();
    document();
  }
  if (mode==255) Configmode();
}
//------------------------------------------ start
void start()
{
    Serial.println(F("\e[2J\e[1;1HASAR, the Analog Servo controller with ARduino"));
    Serial.println(versie);

    decnum=EEPROM.read(499);
    if (decnum>100) 
    {
      Serial.println(F("\nSpecify number for this ASAR"));
      decnum=getint(-1,1,100);
      EEPROM.write(499,decnum);
      dwit();
      Serial.print(F("\nThis is your ASAR number "));
      Serial.println(decnum);
      dgeel();
      quick=true;
    } 
    if (mode==0) 
    {
      Serial.print(F("\nStarting Normal mode of ASAR #"));
      Serial.println(decnum);
      Serial.println(F("\nPress save button to save servo positions"));
      Serial.println(F("Connect USB cable and press 'C' for configuration mode"));
    }
    if (mode==255) 
    {
      Serial.print(F("\nStarting configuration of ASAR #"));
      Serial.println(decnum);
    }
    Serial.println("");
    digitalWrite (LED_BUILTIN,mode==255);

    changes=false;
    pinsel=false;
}
//==========================================================
void Configmode() 
{
const String gencmds="PTDM?ESC";
//const String gencmds="PRTDM?ESC";
  do
  {
    Serial.println(F("\nPlease specify action"));
    Serial.print(F("P/T/D/M/C/?/E/S: "));
    gencmd=getcmd();
    if (gencmds.indexOf(gencmd)>=0) 
    {
      if (gencmd=='C') resetall();
      if (gencmd=='D') document();
      if (gencmd=='T') testall();
      if (gencmd=='M') defaultspeed();
      if (gencmd=='?') showhelp();
      if (gencmd=='R') defroutes();
//      if (gencmd=='W') getRG();
      if (gencmd=='S') testsw();
      if (gencmd=='P') processpin();
    } 
    else showinvalid();
  } 
  while (gencmd!='E');

  Serial.print(F("\nAre you sure you want to start normal mode? Y/N "));
  if (askyn()=='Y')
  {
    EEPROM.write(500,0);  //Force normal mode after reset
    delay(200);
//    asm volatile ("  jmp 0");  //herstart
  mode=0;
  digitalWrite (LED_BUILTIN,LOW);
  start();
  }
}

//---------------------------- quickconfig
void quickconfig()
{
boolean frogyn=false;
boolean ledyn=false;
boolean nummers=false;
byte maxservo=12;

  Serial.print(F("\nDo you want frogpoint polarisation for ALL servo's ? Y/N: "));
  if (askyn()=='Y') frogyn=true;  
  Serial.print(F("\nDo you want led indicators for ALL servo's ? Y/N: "));
  if (askyn()=='Y') ledyn=true;  
  Serial.print(F("\nDo you want to assign administrative numbers ? Y/N: "));
  if (askyn()=='Y') nummers=true; 
  if (frogyn || ledyn) maxservo=8;
  if (frogyn && ledyn) maxservo=5;
  Serial.println(F("\nHow many servo's do you want to configure?"));
  maxservo=getint(-1,1,maxservo);
  for (int n=0; n<maxservo; n++)
  {
    curpin=n+2;
    if (curpin==13) curpin=14;
    allpins[curpin].zwitch=n+1;    //assign switch
    switches[n]=curpin;
    allpins[curpin].type=2;
    allpins[curpin].Langle=75;
    allpins[curpin].Hangle=105;
    allpins[curpin].swrelay=90;
    allpins[curpin].mspeed=defspeed;
    allpins[curpin].nummer=0;
    allpins[curpin].frogpin=255;
    allpins[curpin].ledpin=255;
    allpins[curpin].buddy=255;
    allpins[curpin].RG='X';
    servpin=ip[curpin];
    bounceangles[servpin]=0;
//    Serial.println(servpin);
    if (frogyn) getfrog(false);
    if (ledyn) getled(false);

    if (nummers) getnummer();
    savepin();
  }
  document();
  quick=false;
}
//---------------------------- processpin
void processpin()
{
  curpin=getpin();
  pinsel=true;
  if (allpins[curpin].type==255) //undefined pin
  {
    allpins[curpin].type=2;
    newservo();
    servopin();
  }
  else //already defined pin --> changes
  if (allpins[curpin].type==2)      // Existing servo
  {
    Servos[servpin].attach(curpin);
    Serial.println("\e[37m\e[1m");
    showservo(curpin);
    Serial.println("");
    allpins[curpin].angle=allpins[curpin].swrelay;
    allpins[curpin].output=1; // to low
    setanglen(curpin);
    showangle(curpin);
    servopin();
  }
  Servos[servpin].detach();
  if (changes) savepin();
}
//------------------------------ showangle
void showangle(int n)
{
  dwit();
  if (allpins[curpin].angle==allpins[curpin].Langle) 
  {
    Serial.print(F("Angle set to LOW: "));
    cmdLH[curpin] = 'L';
  }
  else //if (allpins[curpin].angle==allpins[curpin].Hangle) 
  {
    Serial.print(F("Angle set to HIGH: "));
    cmdLH[curpin] = 'H';
  }
  Serial.println(allpins[curpin].angle);
  dgeel();
}
//------------------------------ testsw
void testsw()
{
  Serial.println(F("\nPress the buttons randomly."));
  Serial.println(F("Press save button to stop testing."));
  do
  {
    zero=true;
    if (millis()>lastswmillis+150)  //to cancel bouncing
    {
      zwitch=getswitches(19);
      if (zwitch>0 && zero) 
      {
        spin=switches[zwitch-1];
        Serial.print(F("Switch "));
        Serial.print(zwitch);
        Serial.println(F(" pressed"));
        zero=false;
        delay(200);
      } 
      if (zwitch==0 && !zero) zero=true;
      lastswmillis=millis();
    }
  } 
  while (digitalRead(0)==HIGH);
}
//------------------------------- defbounce
void defbounce()
{
int nr;

  Serial.print(F("\nSpecify bounce angles for servo on pin "));
  Serial.println(curpin);
  nr = getint(bounceangles[servpin],0,9999);

  if (bounceangles[servpin] != nr) changes=true;
  if (!quick)
  {
    dwit();
    Serial.print(F("\nBounce angles set to "));
    Serial.println(nr);
    dgeel();
  }
  bounceangles[servpin]= nr;
  cvtbounce();
}
//------------------------------ cvtbounce
void cvtbounce()
{
byte i;
byte id=0;
int m;
int bounce;
    bounce=bounceangles[servpin];
    for (i=0;i<4;i++)
    {
      bounces[servpin][i]=0;
      m=ten[i];
      if (bounce>=m)
      {
        bounces[servpin][id]=bounce/m;
        bounce=bounce-(bounces[servpin][id]*m);
        id++;
      }
    } 
//    for (byte n=0;n<4;n++) Serial.println (bounces[servpin][n]);
}
//------------------------------ GetRG
void getRG()
{
  for (byte n=2; n<15; n++) if (allpins[n].type==2) //set LOW
  {
    allpins[n].angle=allpins[n].Langle+5;
    allpins[n].output=1;
    Servos[servpin].attach(n); 
    Serial.println("");
    setanglen(n);
    Serial.println(F("What is the position of this servo ?"));
//    Serial.print(allpins[n].nummer);
//    Serial.print(F(" on pin "));
//    Serial.print(n);
//    Serial.print(F(" and switch "));
//    Serial.println(allpins[n].zwitch);
    do 
    {
      Serial.print(F("Straight(G) or Round(R): "));
      gencmd=getcmd(); 
    } while (gencmd!='G' && gencmd!='R');  
    allpins[n].RG=gencmd;
    EEPROM.write(9+11*n,allpins[n].RG);
    Servos[servpin].detach();
  }
}
//------------------------------ defroutes
void defroutes()
{
byte numwissel;
  numservos=0;
  for (byte n=2;n<15;n++) if (allpins[n].type==2) numservos++; //tel aantal servo's
  if (numservos==0)
  {
    dblauw();
    Serial.println(F("\nNo servo's defined or no directions specified!"));
    dgeel();
    return;
  }

  for (byte n=2;n<15;n++) if (allpins[n].type==2 && allpins[n].RG=='X')
  {
    dblauw();
    Serial.println(F("\nNo directions specified!"));
    dgeel();
    n=15;
    Serial.print(F("\nDo you want to specify them now? Y/N: "));
    if (askyn()=='Y') getRG(); else return;
  }
  
  do 
  {
    Serial.println(F("\nSpecify route number"));
    route=getint(0,1,12);  //vraag route
    if (route>0)   //als geen enter
    {
      Serial.println("");
      if (routes[route-1][0]==0) newroute(); 
      numwissel=showroute();  // show route and return number of turnours in this route
      changeroute(numwissel);
    }
  } while (route>0);
  Serial.println("");
}
//----------------------------- changeroute
void changeroute(byte numwissel)
{
int pos;
;  if (numwissel>12) numwissel=12;
  do
  {
    Serial.println(F("\nSpecify the position to change "));
    Serial.print(F("(1-"));
    Serial.print(numwissel);
    Serial.print(F(", 99=remove route, enter=save): "));
    pos=getnum();
//    Serial.println(pos);
    if (pos==99 || (pos>=0 && pos<=numwissel))
    if (pos>0 && pos!=99)
    {
      askrg(pos);
      numwissel=showroute();
    }
  } while (pos>0 && pos!=99);
  if (pos==-1) saveroute();
  if (pos==99)  //remove whole route
  {
    for (byte n=0;n<12;n++)
    {
      routes[route-1][n]=0;
      if (EEPROM.read(600+n+12*(route-1))!=255) EEPROM.write(600+n+12*(route-1),255);
    }
    Serial.print(F("\nRoute "));
    Serial.print(route);
    Serial.println(F(" has been removed"));
  }
}
//------------------------------
void newroute()
{
byte prg=1;
byte ret;
  do 
  {
    ret=askrg(prg);
//    Serial.println(ret);
    if (ret==0) prg=numservos+1; else prg++;
 
  } while (prg<=numservos); 
   
// save data

//  saveroute();

}
//---------------------------- saveroute
void saveroute()
{
byte ipin;
    for (byte n=0; n<12; n++)
    {
      ipin=routes[route-1][n];
      if (ipin>0) EEPROM.write (600+n+12*(route-1),ipin); else n=12;
    }
    dwit();
    Serial.print(F("\nRoute settings are saved for route "));
    Serial.println(route);
    dgeel();
}
//---------------------------- askrg
byte askrg(byte prg)
{
String str;
int ipin;
char rg;
boolean ok;
  do
  {
    str="";  rg=' '; ipin=0; ok=true;
    Serial.print(F("\nSet pin and direction for position "));
    Serial.print(prg);
    Serial.print(" (ppR or ppG): ");
//      str=Serial.readStringUntil('\n');
//      str.toUpperCase();
    str=oneroute();
    rg=str.charAt(str.length()-2);
    ipin=str.toInt();

    if ((rg=='R' || rg=='G') && allpins[ipin].type==2)  //ingave ok
    {
      dwit();
      Serial.print(F("\nTurnout on pin "));
      Serial.print(ipin);
      Serial.print(F(" set to "));
      if (rg=='R') 
      {
        Serial.println("Rounded"); 
        ipin=10*ipin;
      } 
      else Serial.println("Straight");
      dgeel();
      routes[route-1][prg-1]=ipin;
    }
    else if (ipin>0)
    {
      dblauw();
      Serial.println(F("\nInvalid input!"));
      dgeel();
      ok=false;
    } 
  } while (!ok);
  return ipin;
}
//---------------------------- oneroute
String oneroute()
{
String str;
int inChar;
int nr;

//    while (Serial.available()) Serial.read();
    flushwait();
    str="";
    inChar=0;
    nr=0;
    while (inChar != 13)
    {
      if (Serial.available()) 
      {
        inChar = Serial.read();       
        if (inChar==127)  //Backspace
        {
          str = str.substring(0, str.length() - 1);
          nr--;
        }
        str += (char)inChar;
        nr++;
      }
    }
    str.toUpperCase();
    if (nr==0) return "0X"; else return str;
}
//-----------------------------
byte showroute()
{
byte prg=0;
byte pin;
char rg;
byte nw=0;
  Serial.print("Route ");    
  Serial.print(route);
  Serial.print(": ");    
  while (prg <13)
  {
    pin=routes[route-1][prg];
    if (pin>=20) 
    {
      pin=pin/10;
      rg='R';
    } else rg='G';
    if (pin>0)
    {
      Serial.print(pin);
      Serial.print(rg);
      Serial.print(" "); 
      nw++;
    } else prg=12;
    prg++;  
  }
  Serial.println(" "); 
  if (numservos==nw) return nw; else return nw+1;
}

//============================== defaultspeed
void defaultspeed()
{
int curval;
    curval=defspeed;
    Serial.println(F("\nSet default move speed for all servo's"));
    defspeed = getint(curval,5,50);
    dwit();
    Serial.print(F("\nDefault speed set to "));
    Serial.print(defspeed); 
    Serial.println(F(" ms/step ")); 
    dgeel();  
    EEPROM.write(230,defspeed);
    for (int n=2;n<maxpins;n++) if (allpins[n].mspeed==curval) allpins[n].mspeed=defspeed;
}
//--------------------------- movespeed
int movespeed()
{
int curval;
    curval=allpins[curpin].mspeed;
    Serial.print(F("\nSet move speed for pin "));
    Serial.println(curpin);
    allpins[curpin].mspeed = getint(curval,5,50);
    dwit();
    Serial.print(F("\nSpeed set to "));
    Serial.print(allpins[curpin].mspeed); 
    Serial.println(F(" ms/step ")); 
    dgeel();  
    if (allpins[curpin].mspeed != curval) changes=true;
}
//----------------------------- savepin 
void savepin()
{
    EEPROM.write(1+11*curpin,allpins[curpin].Langle);
    EEPROM.write(2+11*curpin,allpins[curpin].Hangle);
    EEPROM.write(3+11*curpin,allpins[curpin].mspeed);  
    EEPROM.write(4+11*curpin,allpins[curpin].frogpin);
    EEPROM.write(5+11*curpin,allpins[curpin].zwitch);
    EEPROM.write(6+11*curpin,allpins[curpin].buddy);
    EEPROM.write(7+11*curpin,highByte(allpins[curpin].nummer));
    EEPROM.write(8+11*curpin,lowByte(allpins[curpin].nummer));
    EEPROM.write(9+11*curpin,allpins[curpin].RG);
    if (allpins[curpin].frogpin < 255)
    {   
      if (allpins[curpin].type==2) EEPROM.write(10+11*allpins[curpin].frogpin,4);
      EEPROM.write(4+11*allpins[curpin].frogpin,curpin);
    } 
    if (allpins[curpin].ledpin < 255)
    {   
      if (allpins[curpin].type==2) EEPROM.write(10+11*allpins[curpin].ledpin,3);
      EEPROM.write(11+11*allpins[curpin].ledpin,curpin);
    }     
    EEPROM.write(10+11*curpin,allpins[curpin].type);
    EEPROM.write(11+11*curpin,allpins[curpin].ledpin);
//    if (allpins[curpin].type==2)
//    {
//      EEPROM.write(300+2*servpin,highByte(bounceangles[servpin]));
//      EEPROM.write(301+2*servpin,lowByte(bounceangles[servpin]));
//    }
    changes=false;
    if (!quick)
    {
      dwit();
      Serial.println(F("\nPin settings are saved!"));
      dgeel();
    }
/*
    for (byte i=1; i<=250; i++) 
    {
      Serial.println(EEPROM.read(i));
      if ((i/11)*11==i) Serial.println("---------------");
    }
*/
  }
  //---------------------------- getcmd
char getcmd()
{
  flushwait();
  byte Command = Serial.read();
  if (Command==13) // Just enter
  {
    pinsel=false;
    Command='X';
  }
  else if (Command>95) Command=Command-32; // to capitals
 
  Serial.println ("");
  return Command;
}
//--------------------------- newservo
void newservo()
{
  Serial.println(F("\nPin is undefined. A switch number will be assigned first"));
  allpins[curpin].zwitch = 255;
  getswitch();
  allpins[curpin].mspeed = defspeed;
  allpins[curpin].Langle=75;
  allpins[curpin].Hangle=105;
  allpins[curpin].swrelay=90;
  allpins[curpin].nummer=0;
  allpins[curpin].frogpin=255;
  allpins[curpin].ledpin=255;
  allpins[curpin].buddy=255;
  allpins[curpin].RG='X';
  bounceangles[servpin]=0;
  Servos[servpin].attach(curpin);
  allpins[curpin].output=1;
  setanglen(curpin);
}
//---------------------------- getswitch
void getswitch()
{
int sw;
int curval;
  curval=allpins[curpin].zwitch;
  do
  {
    Serial.println(F("What switch number will be used ?"));
    if (curval==255) sw=getint(-1,1,12); else sw=getint(curval,1,12);
    if (switches[sw-1]>0 && sw!=curval) 
    {
      dblauw();
      Serial.print(F("\nSwitch "));
      Serial.print(sw);
      Serial.print(F(" is in use for servo "));
      Serial.print(allpins[switches[sw-1]].nummer);
      Serial.print(F(" on pin "));
      Serial.println(switches[sw-1]);
      dgeel();
      sw=0;
    }
  }
  while (sw==0);
  
  switches[sw-1]=curpin;
  allpins[curpin].zwitch=sw;
  if (curval!=sw) changes=true;
  dwit();
  Serial.print(F("\nSwitch "));
  Serial.print(sw);
  Serial.print(F(" is assigned to servo "));
  Serial.print(allpins[switches[sw-1]].nummer);
  Serial.print(F(" on pin "));
  Serial.println(switches[sw-1]);
  dgeel();
}
//---------------------------- resetall
void resetall()
{
  Serial.print(F("\nAre you sure you want to clear all values? Y/N: "));
  if (askyn()=='Y')
  {
    Serial.print(F("Are you absolutely sure? Y/N: "));
    if (askyn()=='Y')
    {
      for (int n=0; n<maxpins; n++) 
      {
        if (EEPROM.read(1+11*n)!=255) EEPROM.write(1+11*n,255);
        if (EEPROM.read(2+11*n)!=255) EEPROM.write(2+11*n,255);
        if (EEPROM.read(3+11*n)!=255) EEPROM.write(3+11*n,255);
        if (EEPROM.read(4+11*n)!=255) EEPROM.write(4+11*n,255);
        if (EEPROM.read(5+11*n)!=255) EEPROM.write(5+11*n,255);
        if (EEPROM.read(6+11*n)!=255) EEPROM.write(6+11*n,255);        
        if (EEPROM.read(7+11*n)!=255) EEPROM.write(7+11*n,255);
        if (EEPROM.read(8+11*n)!=255) EEPROM.write(8+11*n,255); 
        if (EEPROM.read(9+11*n)!='X') EEPROM.write(9+11*n,'X');
        if (EEPROM.read(10+11*n)!=255) EEPROM.write(10+11*n,255);       
        if (EEPROM.read(11+11*n)!=255) EEPROM.write(11+11*n,255);       
      }
      for (byte r=0; r<12; r++)
      {
        for (byte w=0; w<12; w++) 
        {
          if (EEPROM.read(600+w+(12*r))!=255) EEPROM.write(600+w+(12*r),255);
//          serial.print routes[r][w]=0;
        }
      }
 /*
      for (byte n=0; n<12; n++)
      {
        if (EEPROM.read(300+2*n)!=255) EEPROM.write(300+2*n,255);
        if (EEPROM.read(301+2*n)!=255) EEPROM.write(301+2*n,255);
      }
*/
      EEPROM.write(499,255);
      delay(500);
      asm volatile ("  jmp 0");  //herstart 
    }
  }
}
//---------------------------- document
void document()
{
boolean br=false;
  Serial.print(F("\nSettings of ASAR number "));
  Serial.println(decnum);
  Serial.print(F("\nDefault servo speed: "));
  Serial.print(defspeed);
  Serial.println(F(" ms/step"));
  Serial.println("");

  for (int n=2;n<maxpins;n++)
  {
    if (n!=13)
    {
      if (n<10) Serial.print(F("Pin  ")); else Serial.print(F("Pin "));
      Serial.print(n); 
      Serial.print(": ");
      if (allpins[n].type==255) Serial.print(F("not used"));
      if (allpins[n].type==2) showservo(n);    
      if (allpins[n].type==3) showled(n);
      if (allpins[n].type==4) showcompfrog(n);    
      Serial.println("");
    }
  }
/*
  Serial.println("");
  for (route=1;route<13;route++) if (routes[route-1][0]>0) 
  {
    showroute();
    br=true;
  }
  if (!br) Serial.println(F("No routes defined"));
*/  
}
//---------------------------- showcompfrog
void showcompfrog(int n)
{
  Serial.print(F("Frog pin for pin "));
  Serial.print(allpins[n].frogpin);
}
//---------------------------- showled
void showled(int n)
{
  Serial.print(F("Led pin for pin "));
  Serial.print(allpins[n].ledpin);
}
//---------------------------- showservo
void showservo(int n)
{
  Serial.print(F("Servo #"));
  Serial.print(allpins[n].nummer);
  Serial.print(F(" on switch "));
  Serial.print(allpins[n].zwitch);
  Serial.print(F(", Angles "));
  Serial.print(allpins[n].Langle);
  if (allpins[n].RG!='X')
  {
    Serial.print("(");
    Serial.print(allpins[n].RG);
    Serial.print(")");
  }
  Serial.print(F("/"));
  Serial.print(allpins[n].Hangle);
  Serial.print(F(", Frog "));
  if (allpins[n].frogpin<255) 
       Serial.print(allpins[n].frogpin);
  else Serial.print(F("no"));
  Serial.print(F(", Led "));
  if (allpins[n].ledpin<255) 
       Serial.print(allpins[n].ledpin);
  else Serial.print(F("no"));
  Serial.print(F(", Speed "));
  Serial.print(allpins[n].mspeed);
  Serial.print(F(", Buddy "));
  if (allpins[n].buddy<255) 
       Serial.print(allpins[n].buddy);
  else Serial.print(F("no"));
//  Serial.print(F(", Bounce "));
//  if (bounceangles[ip[n]]>0) 
//       Serial.print(bounceangles[ip[n]]);
//  else Serial.print(F("no"));
}
//---------------------------- testall
void testall()
{ 
  Serial.println("");
  if (!pinsel)      //alles
  {
    for (int n=2;n<15;n++) if (allpins[n].type==2) testservo(n);
    else
    {
      Serial.print(F("On pin "));
      Serial.print(n);
      Serial.println(F(" is no servo defined"));
    }
  }
  if (pinsel && allpins[curpin].type==2) testservo(curpin);
  
  if (allpins[curpin].type==2 && pinsel) Servos[servpin].attach(curpin);
}
//--------------------------- testservo
void testservo(int n)
{
byte sa;
  Serial.print(F("Testing servo "));
  Serial.print(allpins[n].nummer);
  Serial.print(F(" on pin "));
  Serial.println(n);
  Servos[servpin].attach(n);
  sa=allpins[n].output;
//  if (cmdLH[n]=='L') allpins[n].output=allpins[n].mspeed;
//  if (cmdLH[n]=='H') allpins[n].output=1-allpins[n].mspeed;
  allpins[n].output=1-allpins[n].output;
  for (byte i=0;i<4;i++)
  {
    setanglen(n);
    allpins[n].output=1-allpins[n].output;
  }
  if (n!=curpin) Servos[servpin].detach();
  allpins[n].output=sa;
}
//---------------------------- getpin
int getpin()
{
int cpin;

  valid = false;
  while (!valid)
  {
    Serial.print(F("\nSpecify pin number (2-12,14): "));
//    cpin = getint(-1,2,13);
    cpin=getnum();
    if ((cpin>=2 && cpin<=12) || cpin==14) //valid number
    {
      if (allpins[cpin].type==3 || allpins[cpin].type==4)
      {
        dblauw();
        Serial.print(F("\nPin "));
        Serial.print(cpin);
        if (allpins[cpin].type==3) 
        {
          Serial.print(F(" is already in use as led pin for pin "));
          Serial.println(allpins[cpin].ledpin);
        }
        if (allpins[cpin].type==4) 
        {
          Serial.print(F(" is already in use as frog pin for pin "));
          Serial.println(allpins[cpin].frogpin);
        }
        dgeel();
      } 
      else valid=true;
    } 
    else
    {
      if (cpin != -1) 
      { 
         dblauw();
         Serial.println(F("\nInvalid pin number!"));
         dgeel();
      } else valid=true;
    }
  }
  servpin=ip[cpin];
  return cpin;
}

//---------------------------- servopin
void servopin()
{
const String servcmds="N9+-CF?XDTLBMS";
// const String servcmds="N9+-CF?XDATLRBMS";
  do
  {
    Serial.print(F("\nSpecify action for Servo on pin "));
    Serial.println (curpin);
    Serial.print(F("N/9/+/-/C/F/D/T/L/R/B/M/S/A/?/enter: "));
    gencmd=getcmd();
    if (servcmds.indexOf(gencmd)>=0) 
    {
      if (gencmd=='N') getnummer();
      if (gencmd=='9') setto90();
      if (gencmd=='+') countup();
      if (gencmd=='-') countdown();
      if (gencmd=='C') changeangle();
      if (gencmd=='S') getswitch();
      if (gencmd=='F') getfrog(true);
      if (gencmd=='M') movespeed();
      if (gencmd=='D') document();
      if (gencmd=='R') resetpin();
      if (gencmd=='L') getled(true);
      if (gencmd=='B') getbuddy();
      if (gencmd=='A') defbounce();  //abprallen
      if (gencmd=='T') testservo(curpin);
      if (gencmd=='?') showhelp();
    } else showinvalid();
  } 
  while (gencmd != 'X');
}
//==================================
void showinvalid()
{
  dblauw();
  Serial.print(F("Invalid command!!"));
  Serial.println("\e[33m\e[1m");
}
//---------------------------------- changeangle
void changeangle()
{
//  if (cmdLH[curpin]=='L') allpins[curpin].output=allpins[curpin].mspeed;
//  if (cmdLH[curpin]=='H') allpins[curpin].output=1-allpins[curpin].mspeed;
  allpins[curpin].output=1-allpins[curpin].output;
  setanglen(curpin); 
  showangle(curpin);
}
//---------------------------------- showhelp
void showhelp()
{
  if (allpins[curpin].type==2 && pinsel)
  {
    Serial.println(F("9: 90 degrees"));  
    Serial.println(F("+: Increment angle"));
    Serial.println(F("-: Decrement angle"));
    Serial.println(F("C: Change to other angle"));
    Serial.println(F("N: Number"));
    Serial.println(F("F: Set Frog pin"));
    Serial.println(F("L: Set Led pin"));
    Serial.println(F("R: Reset pin"));
    Serial.println(F("B: Set Buddy pin"));
//    Serial.println(F("A: Stuiter hoeken"));
    Serial.println(F("M: Move speed this servo"));
    Serial.println(F("S: Switch number"));
    Serial.println(F("T: Test this servo"));
    Serial.println(F("enter: To generic commands"));   
  }
  else
  {
    Serial.println(F("P: Select pin"));
    Serial.println(F("C: Clear(=reset) ASAR"));
    Serial.println(F("T: Test all servo's"));
    Serial.println(F("M: Set default Move speed"));
    Serial.println(F("S: test Switches"));
//    Serial.println(F("R: define Routes"));
    Serial.println(F("E: Exit to normal mode"));
  }
  Serial.println(F("D: Show all settings"));
}
//---------------------------------- setto90
void setto90()
{
   allpins[curpin].angle=90;
   Servos[servpin].write(allpins[curpin].angle);  //without timer!
   dwit();
   Serial.println(F("Angle set to 90 degrees"));
   dgeel();
}
//---------------------------------- countup
void countup()
{
  if (cmdLH[curpin] == 'H') 
  {
    allpins[curpin].Hangle++;
    allpins[curpin].angle=allpins[curpin].Hangle;
  } 
  else if (cmdLH[curpin] == 'L' && allpins[curpin].Langle < allpins[curpin].Hangle-5) 
  {
    allpins[curpin].Langle++;
    allpins[curpin].angle=allpins[curpin].Langle;
  }
  else 
  { 
    dblauw();
    Serial.println(F("LOW angle to HIGH"));
    dgeel();
  }
  allpins[curpin].swrelay=(allpins[curpin].Hangle+allpins[curpin].Langle)/2;
  Servos[servpin].write(allpins[curpin].angle);  //without timer!
  changes=true;
  dwit();
  if (cmdLH[curpin] == 'H') Serial.print(F("HIGH angle set to "));
  if (cmdLH[curpin] == 'L') Serial.print(F("LOW angle set to "));
  Serial.println(allpins[curpin].angle);
  dgeel();
}
//---------------------------------- countdown
void countdown()
{
  if (cmdLH[curpin] == 'L') 
  {
    allpins[curpin].Langle--;
    allpins[curpin].angle=allpins[curpin].Langle;
  }
  else if (cmdLH[curpin] == 'H' && allpins[curpin].Hangle > allpins[curpin].Langle+5) 
  {
    allpins[curpin].Hangle--;
    allpins[curpin].angle=allpins[curpin].Hangle;
  } 
  else 
  { 
    dblauw();
    Serial.println(F("HIGH angle to LOW"));
    dwit();
  }
  allpins[curpin].swrelay=(allpins[curpin].Hangle+allpins[curpin].Langle)/2;
  Servos[servpin].write(allpins[curpin].angle);  //without timer!
  changes=true;  
  dwit();
  if (cmdLH[curpin] == 'H') Serial.print(F("HIGH angle set to "));
  if (cmdLH[curpin] == 'L') Serial.print(F("LOW angle set to "));
  Serial.print(allpins[curpin].angle);
  Serial.println(F(" degrees"));
  dgeel();
}
//---------------------------------- getbuddy
void getbuddy()
{
int curbud;
int newbud;
  curbud=allpins[curpin].buddy;
  Serial.println(F("Please enter a Buddy pin number"));
  if (curbud==255) 
  {
    Serial.print(F("Enter a value from 2-12, 14: "));
    newbud=getnum();
    if (newbud==-1) return;
  }
  else 
  {
    Serial.print(F("Enter a value from 2-12, 14 or 99 to remove: "));
    newbud=getnum();
    if (newbud==-1) newbud=curbud;
  }

  if (newbud==99) 
  {
    allpins[curpin].buddy=255;
    dwit();
    Serial.println(F("\nBuddy pin removed."));
    changes=true;
    dgeel();
  }
  else
  {
    if (allpins[newbud].type==2)
    {
      if (newbud!=curpin && newbud!=curbud)
      {
        allpins[curpin].buddy=newbud;
        changes=true;
        dwit();
        Serial.print(F("\nPin "));
        Serial.print(newbud);
        Serial.println(F(" set as buddy pin"));
        dgeel();
      }
    }
    else
    {
      dblauw();
      Serial.print(F("\nNo servo defined for pin "));
      Serial.println(newbud);
      dgeel();
    }
  }
}
//---------------------------------- getnummer
void getnummer()
{
int nr;
int snum;
  valid=false;
  while(!valid)
  {
    Serial.print(F("\nSpecify turnout number on pin "));
    Serial.println(curpin);
    nr = getint(allpins[curpin].nummer,0,2000);
    if (nr>0) 
    {
      snum = chknum(nr);
      if (snum>0) 
      {       
        Serial.print(F("\nNumber in use for servo on pin "));
        Serial.println(snum);
        Serial.print(F("Is that OK ? (Y/N, N=default): "));
        if (askyn() == 'Y') valid=true;
      } else valid=true;
    } else valid=true;
  }
  if (allpins[curpin].nummer != nr) changes=true;
  if (!quick)
  {
    dwit();
    Serial.print(F("\nNumber set to "));
    Serial.println(nr);
    dgeel();
  }
  allpins[curpin].nummer= nr;
}
//---------------------------------- getfrog
void getfrog(boolean ask)
{
int n;
int curfp;
boolean frogyn=true;
  curfp=allpins[curpin].frogpin;
  if (curfp==255)
  {
    if (ask)
    {
      Serial.print(F("\nDo you want frogpoint polarisation for servo on pin "));
      Serial.print(curpin);
      Serial.print(F(" and switch "));
      Serial.print(allpins[curpin].zwitch);
      Serial.print(F(" ? Y/N "));
      if (askyn()!='Y') frogyn=false;
    } 
    if (frogyn)
    {
      for (n=18;n>1;n--)
      {
        if (allpins[n].type==255 && n!=13 )
        {
          allpins[curpin].frogpin=n;
          allpins[n].frogpin=curpin;
          allpins[n].type=4;
          if (!quick)
          {
            dwit();
            Serial.print(F("Pin "));
            Serial.print(n);
            Serial.print(F(" is assigned as frogpin for servo on pin "));
            Serial.print(curpin);          
            Serial.print(F(" and switch "));
            Serial.println(allpins[curpin].zwitch);
            dgeel();
          }
          n=1;
          changes=true;
        }
      }
      if (n==1) nopins();
    }
  }
  else
  {
    Serial.print(F("\nDo you want to remove frogpoint polarisation ? Y/N "));
    if (askyn()=='Y') 
    {
      clearfc(curfp);
      dwit();
      Serial.println(F("Frogpin removed"));
      dgeel();
      changes = true;
    }
  }
}
//---------------------------- nopins
void nopins()
{
  dblauw();
  Serial.println(F("Sorry, no more free pins!"));
  dgeel();
}
//----------------------- getled
void getled(boolean ask)
{
int n;
int curfp;
boolean ledyn=true;
  curfp=allpins[curpin].ledpin;
  if (curfp==255)
  {
    if (ask)
    {
      Serial.print(F("\nDo you want a led indication for servo on pin "));
      Serial.print(curpin);
      Serial.print(F(" and switch "));
      Serial.print(allpins[curpin].zwitch);
      Serial.print(F(" ? Y/N "));
      if (askyn()!='Y') ledyn=false;
    }
    if (ledyn)
    {
      for (n=18;n>1;n--)
      {
        if (allpins[n].type==255 && n!=13)
        {
          allpins[curpin].ledpin=n;
          allpins[n].ledpin=curpin;
          allpins[n].type=3;
          if (!quick)
          {
            dwit();
            Serial.print(F("Pin "));
            Serial.print(n);
            Serial.print(F(" is assigned as led pin for servo on pin "));
            Serial.print(curpin);          
            Serial.print(F(" and switch "));
            Serial.println(allpins[curpin].zwitch);
            dgeel();
          }
          n=1;
          changes=true;
        }
      }
      if (n==1) nopins();
    }
  }
  else
  {
    Serial.print(F("\nDo you want to remove the led indication ? Y/N "));
    if (askyn()=='Y') 
    {
      clearled(curfp);
      dwit();
      Serial.println(F("Led indication removed"));
      dgeel();
      changes = true;
    }
  }
}
//------------------------- resetpin
void resetpin()
{
byte remsw;  // te verwijderen Sx
byte hp;
int hs;
  Serial.print(F("\nDo you want to reset pin "));
  Serial.print(curpin);
  Serial.print(F("? Y/N: "));
  if (askyn() == 'Y')
  {  
    allpins[curpin].type=255;
    allpins[curpin].Hangle=255;
    allpins[curpin].Langle=255;
    allpins[curpin].buddy=255;
    switches[allpins[curpin].zwitch-1]=0;  

    allpins[curpin].zwitch=255;      

    if (allpins[curpin].ledpin<19) clearled(allpins[curpin].ledpin);
    if (allpins[curpin].frogpin<19) clearfc(allpins[curpin].frogpin);

    pinsel=false; 
    changes=true;
    gencmd='X';
    dwit();
    Serial.print(F("\nPin "));
    Serial.print(curpin);
    Serial.println(F(" is now reset."));
    dgeel();
  }
}
//------------------------- setanglen
void setanglen(int i)
{
  allpins[i].jip=true;
  savout[i]=1-allpins[i].output;
  do
  { 
    processervo(i);
  } 
  while (!allpins[i].jip);
  if (allpins[i].Langle==allpins[i].angle) cmdLH[i]='L';
  if (allpins[i].Hangle==allpins[i].angle) cmdLH[i]='H';
}
//------------------------- SwitchRelay
void SwitchRelay(int fp)
{
    if (allpins[fp].setpoint==allpins[fp].Hangle) 
    {
      if (allpins[fp].frogpin < 19) digitalWrite(allpins[fp].frogpin,HIGH);    
      if (allpins[fp].ledpin < 19) digitalWrite(allpins[fp].ledpin,HIGH);
    }
    if (allpins[fp].setpoint==allpins[fp].Langle)
    {
      if (allpins[fp].frogpin < 19) digitalWrite(allpins[fp].frogpin,LOW);
      if (allpins[fp].ledpin < 19) digitalWrite(allpins[fp].ledpin,LOW);
    }
}
//---------------------------------- clearfc
void clearfc(int fp)
{
  allpins[fp].type=255;
  EEPROM.write(10+11*fp,255); //reset type
  EEPROM.write(4+11*fp,255);  //reset frogpin
  if (fp==allpins[curpin].frogpin) allpins[curpin].frogpin=255;    //remove eigen fp
  allpins[fp].frogpin=255;
}
//---------------------------------- clearled
void clearled(int fp)
{
  allpins[fp].type=255;
  EEPROM.write(10+11*fp,255); //reset type
  EEPROM.write(11+11*fp,255);  //reset led
  if (fp==allpins[curpin].ledpin) allpins[curpin].ledpin=255;    //remove eigen fp
  allpins[fp].ledpin=255;
}
//------------------------- flushwait 
void flushwait()
{
  while(Serial.available())    //Empty inputbuffer
  {
    Serial.read();
  }
  while (!Serial.available())   //Wait for new input
  {
/*
    currentMillis = millis();
    digitalWrite(LED_BUILTIN, LED13state);
    if (currentMillis - previousMillis >= 200) 
    {
      LED13state = !(LED13state);
      previousMillis = currentMillis;
    }
*/
    delay (100);
  }
}
//-------------------------------- processroute
/*
void processroute()
{
boolean zero;
char rg;

  zero=true;
  if (millis()>lastswmillis+150)  //to cancel bouncing
  {
    route=getswitches();
//    Serial.println(zwitch);
    if (route>0 && zero) 
    {
      if (route>0 and routes[route-1][0]>0)  // bestaat route?
      {
        Serial.print(F("Switch pressed for route"));
        Serial.print(route);
        zero=false;
        for (byte n=0;n<12;n++)
        {
          spin=routes[route-1][n];
          if (spin==0) n=12;
          else
          {
            if (spin>=20) 
            {
              spin=spin/10;
              allpins[spin].output=1;
            } else allpins[spin].output=0;  
            savout[spin]=1-allpins[spin].output  ;
          }
        }
      } 
      if (route==0 && !zero) zero=true;    
      lastswmillis=millis();
    }
  for (int n=2; n<15; n++) if (allpins[n].type==2) processervo(spin);
  }
}
*/
//-------------------------------- processwitch
void processwitch()
{
byte buddy;
byte Command='x';
static boolean drlow;
static boolean zero=true;
unsigned long lmil;

  if (millis()>lastswmillis+150)  //to cancel bouncing
  {
    zwitch=getswitches(19);
    if (zwitch>0 && zero) 
    {
      spin=switches[zwitch-1];
      if (allpins[spin].jip && spin>0)
      {
        Serial.print(F("Switch "));
        Serial.print(zwitch);
        Serial.print(F(" pressed for servo "));
        Serial.print(allpins[spin].nummer);
        Serial.print(F(" on pin "));
        Serial.println(spin);  
        zero=false;
      }
    } else spin=0;
    if (zwitch==0 && !zero) 
    {
      zero=true;
    }
    
    lastswmillis=millis();
  }

  if (spin>0 && allpins[spin].jip)  // inactive button pressed
  { 
    allpins[spin].output=1-allpins[spin].output;
    savout[spin]=1-allpins[spin].output;
    buddy=allpins[spin].buddy;
    if (buddy<19 && allpins[buddy].jip)  //start buddy if not moving
    {
      allpins[buddy].output=1-allpins[buddy].output;
      savout[buddy]=1-allpins[buddy].output;     
    }
  }

  for (int n=2; n<15; n++) if (allpins[n].type==2) processervo(n);

  if (digitalRead(0)==LOW)
  {
    delay(500);
    if (Serial.available()) Command=Serial.read();
    if (Command=='c' || Command=='C')
    {
      EEPROM.write(500,255);     //Force config mode
      Serial.println("\e[DTo config");
      delay (2000);
      mode=255;
    }
//    while (Serial.available()) Serial.read();
    if (drlow && millis()>(lmil+5000)) drlow=false;
    if (!drlow && mode==0)
    {
      for (int i=2; i<15; i++) 
        if (allpins[i].type==2) EEPROM.write(270+i,allpins[i].output);
      for (int i=0;i<3;i++)
      {
        digitalWrite(LED_BUILTIN,HIGH);
        delay (150);
        digitalWrite(LED_BUILTIN,LOW);
        delay (150);
      }
      Serial.println(F("\nServo settings are saved"));  
      drlow=true;
      lmil=millis();
    } 
  }
}
//------------------------------------------- getswitches
int getswitches(byte pinnr)
{
int zwitch;
unsigned int anin; 
    anin=analogRead(pinnr);
    zwitch=map(anin,0,1000,1,13);
    if (zwitch==13) zwitch=0;
    
    return zwitch;
}
//-------------------------------- chknum 
int chknum (int num)
{
  for (int n=0; n<maxpins; n++) 
  {
    if (allpins[n].nummer == num && n  !=  curpin) return n;
  }
  return 0;
}
//---------------------------------- getnum 
int getnum()
{
String inString = "";
int inChar;
int nr=0;

    flushwait();
    inString="";
    inChar=0;
    while (inChar != 13)
    {
      if (Serial.available()) 
      {
        inChar = Serial.read();       
        if (inChar==127)  //Backspace
        {
          inString = inString.substring(0, inString.length() - 1);
          nr--;
        }
        if (isdigit(inChar)) 
        {
           inString += (char)inChar;
           nr++;
        } 
      }
    } 
    if (nr==0) return -1; else return inString.toInt();
}
//---------------------------------- getint
int getint(int curval, int minval, int maxval)
{
String inString;
byte inChar;
int nr;
boolean valid;
  valid=false;
  while (!valid)
  {
    Serial.print(F("Enter value from "));
    Serial.print(minval);
    Serial.print(F(" to "));
    Serial.print(maxval);
    if (curval>0) 
    {
      Serial.print(F(" ("));
      Serial.print(curval);
      Serial.print(F("): "));
    } else Serial.print(F(": "));

    nr=getnum();
    if (nr==-1 && curval>-1) nr=curval; //accept default bij enter
    if ((nr>=minval && nr<=maxval) || (nr==curval && nr!=-1)) valid=true;
    else badnum();
  }
  return nr;
}
/*
//---------------------------------- adjust
void adjust()
{
//  limits[12] = {66,168,262,350,434,515,594,670,746,824,903,980};
unsigned int readval[13]={0,0,0,0,0,0,0,0,0,0,0,0,0};
unsigned int readcur;
unsigned int maxval;
unsigned int dif;

  Serial.println(F("\nFirst have a look at the manual!"));
  Serial.print(F("Do you want to continue? Y/N "));
  if (askyn()=='Y')
  {  
    Serial.println(F("Wait a moment"));
    maxval=count(); 
    readval[12]=maxval;    //count max value
    for (int n=0; n<12 ;n++)
    {
      if (switches[n]>0)
      {
        digitalWrite(LED_BUILTIN,LOW);
        Serial.print(F("Press and hold switch "));
        Serial.print(n+1); 
        Serial.println(F(" until on board led goes on"));
        do
        {
          delay(200);
          readcur=analogRead(A5); //wait until pressed
        } 
        while(readcur>1000);
        
        readval[n]=count();
  
        do
        {
          delay(200);
          readcur=analogRead(A5);
        } 
        while(readcur<1000);  //wait until released
        Serial.print(F("Released, measured value: "));
        Serial.println(readval[n]);
        readval[n+1]=readval[n] + 85;
      } 
      else
      {
        readval[n]=0;
        n=12;
      }
    }
    for (byte n=0;n<11;n++)
    {
      Serial.print(F("Old: "));
      Serial.print(limits[n]);
      if (readval[n+1]>0) limits[n]=(readval[n]+readval[n+1])/2; else limits[n]=maxval;
      Serial.print(F(", New: "));
      Serial.println(limits[n]);
      EEPROM.write(600+2*n,highByte(limits[n]));
      EEPROM.write(601+2*n,lowByte(limits[n]));
    }
    limits[11]=maxval-15;
    EEPROM.write(622,highByte(limits[11]));
    EEPROM.write(623,lowByte(limits[11]));
  }
}

//--------------------------------- count
unsigned int count()
{
unsigned long total=0;
    Serial.println(F("Start measuring"));
    for (byte count=0; count<100; count++)
    {
      total=total+analogRead(A5);
      delay(50);
    }
    digitalWrite(LED_BUILTIN,HIGH); //turn on led  
    return total/100;
}
*/
//---------------------------------- badnum
void badnum()
{
  dblauw();
  Serial.println(F("\nNumber not valid!"));
  dgeel();
}
//---------------------------------- askyn
char askyn()
{
    flushwait();
    byte Command = Serial.read();
    if (Command=='y') Command = 'Y'; else Command=' ';
    Serial.println ("");
    return Command;
}
//------------------------------------ processervo
void processervo (int n)
{
  servpin=ip[n];
  if (savout[n] != allpins[n].output && allpins[n].jip)    //output changed and quiet --> prepare move
  {
    if (allpins[n].output) 
           allpins[n].setpoint=allpins[n].Langle;
      else allpins[n].setpoint=allpins[n].Hangle;
    if (mode==0) 
    {
      Servos[servpin].attach(n);       // Attach servo
      digitalWrite (LED_BUILTIN, HIGH); 
    }
    allpins[n].jip = false;             // indicate the junction is moving    
    savout[n] = allpins[n].output; //save state
  }
  
  if (allpins[n].angle!=allpins[n].setpoint && millis() > allpins[n].lastmillis+allpins[n].mspeed)  //not ready yet
  {
    if (allpins[n].angle < allpins[n].setpoint) allpins[n].angle++;  // Raise angle if required
    if (allpins[n].angle > allpins[n].setpoint) allpins[n].angle--;  // Decrease angle if required 
    Servos[servpin].write(allpins[n].angle);    // Move servo 1 degree towards required setpoint
    if ((allpins[n].frogpin<20 || allpins[n].ledpin<20) && allpins[n].angle==allpins[n].swrelay) SwitchRelay(n);
//    Serial.println(allpins[n].angle);
    if (allpins[n].angle==allpins[n].setpoint)   //klaar
    {
      allpins[n].jip = true;             // Indicate the junction is in position.
      if (mode==0) 
      {
        Servos[servpin].detach();       // Detach to prevent jittering          
        digitalWrite (LED_BUILTIN, LOW);
      }
      Serial.print(F("Servo "));
      Serial.print(allpins[n].nummer);
      Serial.print(F(" on pin "));
      Serial.print(n);
      Serial.print(F(" set to "));
      Serial.print(allpins[n].setpoint);
      Serial.println(F(" degrees"));
      if (allpins[n].setpoint>allpins[n].swrelay) cmdLH[n]='H'; else cmdLH[n]='L';
    } 
    else allpins[n].lastmillis=millis();     // Save last changetime      
  }
}
//====================================
void dwit()
{
  Serial.print("\e[37m\e[1m");
}
//---------------------
void dgeel()
{
  Serial.print("\e[33m\e[1m");
}
//---------------------
void dblauw()
{
  Serial.print(char(7));
  Serial.print("\e[36m\e[1m");
}
//---------------- End Of Program ----------------------

